const fs = require('fs');
const path = require('path');
const yaml = require('yaml');
const logger = require('../utils/logger');

class ClashParser {
  constructor(configPath) {
    this.configPath = configPath;
  }

  /**
   * 解析Clash配置文件
   */
  async parseConfig() {
    try {
      if (!fs.existsSync(this.configPath)) {
        throw new Error(`配置文件不存在: ${this.configPath}`);
      }

      const configContent = fs.readFileSync(this.configPath, 'utf8');
      const config = yaml.parse(configContent);

      if (!config.proxies || !Array.isArray(config.proxies)) {
        throw new Error('配置文件中没有找到有效的代理节点');
      }

      const nodes = [];
      for (const proxy of config.proxies) {
        const node = this.parseProxy(proxy);
        if (node) {
          nodes.push(node);
        }
      }

      logger.info(`成功解析 ${nodes.length} 个节点`, { configPath: this.configPath });
      return nodes;
    } catch (error) {
      logger.error('解析Clash配置文件失败', { error: error.message, configPath: this.configPath });
      throw error;
    }
  }

  /**
   * 解析单个代理节点
   */
  parseProxy(proxy) {
    try {
      const baseNode = {
        name: proxy.name,
        type: proxy.type,
        server: proxy.server,
        port: parseInt(proxy.port)
      };

      switch (proxy.type) {
        case 'ss':
          return this.parseShadowsocks(proxy, baseNode);
        case 'ssr':
          return this.parseShadowsocksR(proxy, baseNode);
        case 'vmess':
          return this.parseVmess(proxy, baseNode);
        case 'trojan':
          return this.parseTrojan(proxy, baseNode);
        case 'http':
        case 'https':
          return this.parseHttp(proxy, baseNode);
        case 'socks5':
          return this.parseSocks5(proxy, baseNode);
        default:
          logger.warn(`不支持的代理类型: ${proxy.type}`, { nodeName: proxy.name });
          return null;
      }
    } catch (error) {
      logger.error('解析代理节点失败', { error: error.message, proxy });
      return null;
    }
  }

  /**
   * 解析Shadowsocks节点
   */
  parseShadowsocks(proxy, baseNode) {
    return {
      ...baseNode,
      password: proxy.password,
      method: proxy.cipher || 'aes-256-gcm'
    };
  }

  /**
   * 解析ShadowsocksR节点
   */
  parseShadowsocksR(proxy, baseNode) {
    return {
      ...baseNode,
      password: proxy.password,
      method: proxy.cipher || 'aes-256-gcm',
      protocol: proxy.protocol || 'origin',
      protocolParam: proxy['protocol-param'] || '',
      obfs: proxy.obfs || 'plain',
      obfsParam: proxy['obfs-param'] || ''
    };
  }

  /**
   * 解析Vmess节点
   */
  parseVmess(proxy, baseNode) {
    const node = {
      ...baseNode,
      uuid: proxy.uuid,
      alterId: parseInt(proxy.alterId) || 0,
      network: proxy.network || 'tcp'
    };

    // 处理TLS配置
    if (proxy.tls) {
      node.tls = true;
      node.sni = proxy.servername || proxy.sni;
    }

    // 处理WebSocket配置
    if (proxy.network === 'ws') {
      node.wsPath = proxy.path || '/';
      if (proxy.headers) {
        node.wsHeaders = JSON.stringify(proxy.headers);
      }
    }

    // 处理HTTP/2配置
    if (proxy.network === 'h2') {
      node.h2Path = proxy.path || '/';
      node.h2Host = proxy.host || proxy.server;
    }

    return node;
  }

  /**
   * 解析Trojan节点
   */
  parseTrojan(proxy, baseNode) {
    const node = {
      ...baseNode,
      password: proxy.password
    };

    // 处理TLS配置
    if (proxy.tls) {
      node.tls = true;
      node.sni = proxy.sni || proxy.server;
    }

    return node;
  }

  /**
   * 解析HTTP代理节点
   */
  parseHttp(proxy, baseNode) {
    const node = {
      ...baseNode,
      username: proxy.username || '',
      password: proxy.password || ''
    };

    // 处理TLS配置
    if (proxy.tls) {
      node.tls = true;
      node.sni = proxy.sni || proxy.server;
    }

    return node;
  }

  /**
   * 解析SOCKS5代理节点
   */
  parseSocks5(proxy, baseNode) {
    return {
      ...baseNode,
      username: proxy.username || '',
      password: proxy.password || ''
    };
  }

  /**
   * 从URL导入配置
   */
  async importFromUrl(url) {
    try {
      const axios = require('axios');
      const response = await axios.get(url, {
        timeout: 10000,
        headers: {
          'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36'
        }
      });

      const configContent = response.data;
      const config = yaml.parse(configContent);

      if (!config.proxies || !Array.isArray(config.proxies)) {
        throw new Error('URL配置中没有找到有效的代理节点');
      }

      const nodes = [];
      for (const proxy of config.proxies) {
        const node = this.parseProxy(proxy);
        if (node) {
          nodes.push(node);
        }
      }

      logger.info(`成功从URL导入 ${nodes.length} 个节点`, { url });
      return nodes;
    } catch (error) {
      logger.error('从URL导入配置失败', { error: error.message, url });
      throw error;
    }
  }
}

module.exports = ClashParser; 