const TelegramBot = require('node-telegram-bot-api');
const logger = require('../utils/logger');
const { Notification } = require('../models');
const { escapeMarkdown } = require('../utils/markdown');

class TelegramNotifier {
  constructor() {
    this.botToken = process.env.TELEGRAM_BOT_TOKEN;
    this.chatId = process.env.TELEGRAM_CHAT_ID;
    this.lastMessageTime = 0; // 用于限流控制
    this.minInterval = 1000; // 最小发送间隔（毫秒）
    
    if (!this.botToken || !this.chatId) {
      logger.warn('Telegram配置不完整，通知功能将被禁用');
      this.enabled = false;
      return;
    }

    try {
      this.bot = new TelegramBot(this.botToken, { 
        polling: false,
        request: {
          timeout: 10000, // 10秒超时
          connect_timeout: 10000
        }
      });
      this.enabled = true;
      logger.info('Telegram通知器初始化成功');
    } catch (error) {
      logger.error('Telegram通知器初始化失败', { error: error.message });
      this.enabled = false;
    }
  }

  /**
   * 发送节点故障通知
   */
  async sendFailureNotification(node, testResult) {
    if (!this.enabled) return;

    const title = '🚨 节点故障通知';
    const message = this.formatFailureMessage(node, testResult);
    
    await this.sendNotification({
      type: 'failure',
      title,
      message,
      nodeId: node.id,
      metadata: {
        nodeName: node.name,
        errorMessage: testResult.errorMessage,
        failureCount: node.failureCount
      }
    });
  }

  /**
   * 发送节点恢复通知
   */
  async sendRecoveryNotification(node, testResult) {
    if (!this.enabled) return;

    const title = '✅ 节点恢复通知';
    const message = this.formatRecoveryMessage(node, testResult);
    
    await this.sendNotification({
      type: 'recovery',
      title,
      message,
      nodeId: node.id,
      metadata: {
        nodeName: node.name,
        latency: testResult.latency,
        speed: testResult.downloadSpeed
      }
    });
  }

  /**
   * 发送订阅测试通知
   */
  async sendSubscriptionNotification(subscription, title, message, summary) {
    if (!this.enabled) return;

    await this.sendNotification({
      type: 'subscription',
      title: escapeMarkdown(title),
      message: message,
      subscriptionId: subscription.id,
      metadata: {
        subscriptionName: subscription.name,
        subscriptionId: subscription.id,
        summary: summary
      }
    });
  }

  /**
   * 发送系统通知
   */
  async sendSystemNotification(title, message, metadata = {}) {
    if (!this.enabled) return;

    await this.sendNotification({
      type: 'system',
      title,
      message,
      metadata: {
        ...metadata,
        timestamp: new Date().toISOString()
      }
    });
  }

  /**
   * 发送测试摘要通知
   */
  async sendSummaryNotification(summary) {
    if (!this.enabled) return;

    const title = '📊 节点测试摘要';
    const message = this.formatSummaryMessage(summary);
    
    await this.sendNotification({
      type: 'summary',
      title,
      message,
      nodeId: null,
      metadata: summary
    });
  }

  /**
   * 发送通知（带重试机制）
   */
  async sendNotification(notificationData) {
    try {
      // 限流控制：确保发送间隔
      const now = Date.now();
      const timeSinceLastMessage = now - this.lastMessageTime;
      if (timeSinceLastMessage < this.minInterval) {
        const waitTime = this.minInterval - timeSinceLastMessage;
        await new Promise(resolve => setTimeout(resolve, waitTime));
      }

      // 将metadata转换为字符串，确保是字符串类型
      const metadataString = typeof notificationData.metadata === 'object' 
        ? JSON.stringify(notificationData.metadata) 
        : String(notificationData.metadata || '');

      const notificationDataForDB = {
        ...notificationData,
        metadata: metadataString
      };

      // 保存通知记录
      const notification = await Notification.create(notificationDataForDB);

      // 发送Telegram消息（带重试）
      const fullMessage = `*${notificationData.title}*\n\n${notificationData.message}`;
      
      const sentMessage = await this.sendMessageWithRetry(fullMessage);

      // 更新通知状态
      await notification.update({
        isSent: true,
        sentTime: new Date()
      });

      this.lastMessageTime = Date.now();

      logger.info('Telegram通知发送成功', {
        notificationId: notification.id,
        messageId: sentMessage.message_id
      });

    } catch (error) {
      logger.error('Telegram通知发送失败', {
        error: error.message,
        notificationData
      });

      // 更新通知状态
      if (notificationData.id) {
        await Notification.update({
          retryCount: notificationData.retryCount + 1,
          errorMessage: error.message
        }, {
          where: { id: notificationData.id }
        });
      }
    }
  }

  /**
   * 带重试的消息发送
   */
  async sendMessageWithRetry(message, maxRetries = 3) {
    for (let attempt = 1; attempt <= maxRetries; attempt++) {
      try {
        const sentMessage = await this.bot.sendMessage(this.chatId, message, {
          parse_mode: 'Markdown',
          disable_web_page_preview: true
        });
        return sentMessage;
      } catch (error) {
        if (attempt === maxRetries) {
          throw error;
        }
        
        // 如果是限流错误，等待更长时间
        if (error.message.includes('429')) {
          const retryAfter = this.extractRetryAfter(error.message) || 30;
          logger.warn(`Telegram API限流，等待 ${retryAfter} 秒后重试`, { attempt });
          await new Promise(resolve => setTimeout(resolve, retryAfter * 1000));
        } else {
          // 其他错误，等待较短时间后重试
          const waitTime = attempt * 2000; // 递增等待时间
          logger.warn(`Telegram发送失败，${waitTime}ms后重试`, { 
            attempt, 
            error: error.message 
          });
          await new Promise(resolve => setTimeout(resolve, waitTime));
        }
      }
    }
  }

  /**
   * 从错误消息中提取重试时间
   */
  extractRetryAfter(errorMessage) {
    const match = errorMessage.match(/retry after (\d+)/);
    return match ? parseInt(match[1]) : null;
  }

  /**
   * 格式化故障消息
   */
  formatFailureMessage(node, testResult) {
    const lines = [
      `*节点名称:* ${node.name}`,
      `*节点类型:* ${node.type}`,
      `*服务器:* ${node.server}:${node.port}`,
      `*连续失败次数:* ${node.failureCount}`,
      `*最后测试时间:* ${new Date().toLocaleString('zh-CN')}`
    ];

    if (testResult.errorMessage) {
      lines.push(`*错误信息:* ${testResult.errorMessage}`);
    }

    if (testResult.testDuration) {
      lines.push(`*测试耗时:* ${testResult.testDuration}ms`);
    }

    return lines.join('\n');
  }

  /**
   * 格式化恢复消息
   */
  formatRecoveryMessage(node, testResult) {
    const lines = [
      `*节点名称:* ${node.name}`,
      `*节点类型:* ${node.type}`,
      `*服务器:* ${node.server}:${node.port}`,
      `*恢复时间:* ${new Date().toLocaleString('zh-CN')}`
    ];

    if (testResult.latency) {
      lines.push(`*延迟:* ${testResult.latency}ms`);
    }

    if (testResult.downloadSpeed) {
      lines.push(`*下载速度:* ${testResult.downloadSpeed} Mbps`);
    }

    if (testResult.ipAddress && testResult.ipAddress !== 'unknown') {
      lines.push(`*IP地址:* ${testResult.ipAddress}`);
    }

    return lines.join('\n');
  }

  /**
   * 格式化摘要消息
   */
  formatSummaryMessage(summary) {
    const lines = [
      `📊 *节点测速总结报告*`,
      `━━━━━━━━━━━━━━━━━━━━`,
      `*测试时间:* ${summary.testTime}`,
      `*总节点数:* ${summary.totalNodes}`,
      `*在线节点:* ${summary.onlineNodes}`,
      `*离线节点:* ${summary.offlineNodes}`,
      `*测试成功率:* ${summary.successRate}%`
    ];

    if (summary.averageLatency) {
      lines.push(`*平均延迟:* ${summary.averageLatency}ms`);
    }

    if (summary.averageSpeed) {
      lines.push(`*平均速度:* ${summary.averageSpeed} Mbps`);
    }



    if (summary.bestNodes && summary.bestNodes.length > 0) {
      lines.push(`\n🏆 *最佳节点 (延迟排序):*`);
      summary.bestNodes.forEach((node, index) => {
        const medal = index === 0 ? '🥇' : index === 1 ? '🥈' : index === 2 ? '🥉' : '•';
        lines.push(`${medal} ${node.name} - ${node.latency}ms`);
      });
    }

    if (summary.failedNodes && summary.failedNodes.length > 0) {
      lines.push(`\n❌ *故障节点:*`);
      summary.failedNodes.forEach(node => {
        lines.push(`• ${node.name}`);
      });
    }

    // 添加测试状态总结
    const statusEmoji = summary.successRate >= 90 ? '🟢' : summary.successRate >= 70 ? '🟡' : '🔴';
    lines.push(`\n${statusEmoji} *测试状态:* ${summary.successRate >= 90 ? '优秀' : summary.successRate >= 70 ? '良好' : '需要关注'}`);

    return lines.join('\n');
  }

  /**
   * 重试失败的通知
   */
  async retryFailedNotifications() {
    try {
      const failedNotifications = await Notification.findAll({
        where: {
          isSent: false,
          retryCount: { [require('sequelize').Op.lt]: 3 }
        },
        order: [['createdAt', 'ASC']]
      });

      for (const notification of failedNotifications) {
        await this.sendNotification({
          id: notification.id,
          type: notification.type,
          title: notification.title,
          message: notification.message,
          nodeId: notification.nodeId,
          retryCount: notification.retryCount
        });

        // 避免发送过快
        await new Promise(resolve => setTimeout(resolve, 1000));
      }

      logger.info(`重试了 ${failedNotifications.length} 个失败的通知`);
    } catch (error) {
      logger.error('重试失败通知时出错', { error: error.message });
    }
  }

  /**
   * 测试连接
   */
  async testConnection() {
    if (!this.enabled) {
      return { success: false, error: '通知器未启用' };
    }

    try {
      const me = await this.bot.getMe();
      logger.info('Telegram连接测试成功', { botName: me.first_name });
      return { success: true, botName: me.first_name };
    } catch (error) {
      logger.error('Telegram连接测试失败', { error: error.message });
      return { success: false, error: error.message };
    }
  }

  /**
   * 获取通知统计
   */
  async getNotificationStats() {
    try {
      const stats = await Notification.findAll({
        attributes: [
          'type',
          'isSent',
          [require('sequelize').fn('COUNT', require('sequelize').col('id')), 'count']
        ],
        group: ['type', 'isSent'],
        raw: true
      });

      return stats;
    } catch (error) {
      logger.error('获取通知统计失败', { error: error.message });
      return [];
    }
  }
}

module.exports = TelegramNotifier; 