const cron = require('node-cron');
const logger = require('../utils/logger');
const SpeedTester = require('./speedTester');
const TelegramNotifier = require('./notifier');
const MultiSubscriptionManager = require('./multiSubscriptionManager');
const { Node, TestResult } = require('../models');
const { escapeMarkdown } = require('../utils/markdown');

class Scheduler {
  constructor() {
    this.speedTester = new SpeedTester();
    this.notifier = new TelegramNotifier();
    this.subscriptionManager = new MultiSubscriptionManager();
    this.isRunning = false;
    this.testInterval = parseInt(process.env.SPEED_TEST_INTERVAL) || 10; // 分钟
    this.failureThreshold = parseInt(process.env.NOTIFICATION_FAILURE_THRESHOLD) || 3;
    this.recoveryThreshold = parseInt(process.env.NOTIFICATION_RECOVERY_THRESHOLD) || 2;
  }

  /**
   * 启动调度器
   */
  start() {
    // 检查是否启用定时测速
    const enableScheduledSpeedTest = process.env.ENABLE_SCHEDULED_SPEED_TEST !== 'false';
    
    if (enableScheduledSpeedTest) {
      logger.info(`启动定时任务调度器 - 定时测速间隔: ${this.testInterval}分钟`);

      // 启用定时测速
      const cronExpression = `*/${this.testInterval} * * * *`;
      this.speedTestJob = cron.schedule(cronExpression, async () => {
        await this.runSpeedTest();
      }, {
        scheduled: true,
        timezone: 'Asia/Shanghai'
      });
    } else {
      logger.info('定时测速功能已禁用');
    }

    // 每小时重试失败的通知
    this.notificationRetryJob = cron.schedule('0 * * * *', async () => {
      await this.notifier.retryFailedNotifications();
    }, {
      scheduled: true,
      timezone: 'Asia/Shanghai'
    });

    // 每天凌晨2点清理旧数据
    this.cleanupJob = cron.schedule('0 2 * * *', async () => {
      await this.cleanupOldData();
    }, {
      scheduled: true,
      timezone: 'Asia/Shanghai'
    });

    // 移除重复的多订阅管理器启动，由应用主文件统一管理
    // this.subscriptionManager.startAutoUpdate();

    logger.info('定时任务调度器启动成功');
  }

  /**
   * 停止调度器
   */
  stop() {
    if (this.speedTestJob) {
      this.speedTestJob.stop();
    }
    if (this.notificationRetryJob) {
      this.notificationRetryJob.stop();
    }
    if (this.cleanupJob) {
      this.cleanupJob.stop();
    }
    // 移除重复的多订阅管理器停止，由应用主文件统一管理
    // this.subscriptionManager.stopAutoUpdate();
    logger.info('定时任务调度器已停止');
  }

  /**
   * 运行速度测试 - 按订阅分组
   */
  async runSpeedTest() {
    if (this.isRunning) {
      logger.warn('上一次测试还在运行中，跳过本次测试');
      return;
    }

    this.isRunning = true;
    const startTime = Date.now();

    try {
      // 获取所有活跃的订阅
      const { Subscription } = require('../models');
      const subscriptions = await Subscription.findAll({
        where: { isActive: true },
        order: [['createdAt', 'ASC']]
      });

      if (subscriptions.length === 0) {
        logger.warn('没有找到活跃的订阅');
        return;
      }

      logger.info(`开始按订阅分组测试，共 ${subscriptions.length} 个订阅`);

      const allResults = [];
      const subscriptionResults = [];

      // 按订阅分组测试
      for (const subscription of subscriptions) {
        try {
          const subscriptionResult = await this.testSubscriptionNodes(subscription);
          subscriptionResults.push(subscriptionResult);
          allResults.push(...subscriptionResult.testResults);
        } catch (error) {
          logger.error(`订阅测试失败: ${subscription.name}`, { error: error.message });
        }
      }

      // 发送每个订阅的单独报告
      for (const result of subscriptionResults) {
        if (result.nodes.length > 0) {
          await this.sendSubscriptionTestSummary(result);
        }
      }

      // 发送总体报告
      await this.sendOverallTestSummary(subscriptionResults);

      const duration = Date.now() - startTime;
      const totalNodes = allResults.length;
      const successCount = allResults.filter(r => r.isSuccess).length;
      
      logger.info(`分组测速完成: ${successCount}/${totalNodes}个成功, 耗时${duration}ms`);

    } catch (error) {
      logger.error('定时速度测试失败', { error: error.message });
      
      // 发送系统错误通知
      await this.notifier.sendSystemNotification(
        '⚠️ 系统错误通知',
        `定时速度测试过程中发生错误:\n\n${error.message}`,
        { error: error.message, timestamp: new Date().toISOString() }
      );
    } finally {
      this.isRunning = false;
    }
  }

  /**
   * 测试单个订阅的所有节点
   */
  async testSubscriptionNodes(subscription) {
    logger.info(`开始测试订阅: ${subscription.name}`);

    // 获取该订阅的所有活跃节点
    const nodes = await Node.findAll({
      where: { 
        subscriptionId: subscription.id,
        isActive: true 
      },
      order: [['name', 'ASC']]
    });

    if (nodes.length === 0) {
      logger.warn(`订阅 ${subscription.name} 没有活跃节点`);
      return {
        subscription,
        nodes: [],
        testResults: [],
        summary: {
          totalNodes: 0,
          onlineNodes: 0,
          offlineNodes: 0,
          successRate: 0,
          averageLatency: null,
          averageSpeed: null
        }
      };
    }

    logger.info(`订阅 ${subscription.name} 有 ${nodes.length} 个节点需要测试`);

    // 批量测试节点
    const testResults = await this.speedTester.testNodes(nodes);

    // 检查是否有高延迟节点需要重测
    const highLatencyNodes = [];
    const highLatencyResults = [];
    
    for (let i = 0; i < nodes.length; i++) {
      const result = testResults[i];
      if (result && result.isSuccess && result.latency && result.latency > 2000) {
        highLatencyNodes.push(nodes[i]);
        highLatencyResults.push(result);
      }
    }

    // 如果有高延迟节点，进行重测
    if (highLatencyNodes.length > 0) {
      logger.info(`订阅 ${subscription.name} 重测 ${highLatencyNodes.length} 个高延迟节点`);
      
      // 重测高延迟节点
      const retestResults = await this.speedTester.testNodes(highLatencyNodes);
      
      // 更新测试结果，使用重测的结果
      for (let i = 0; i < highLatencyNodes.length; i++) {
        const node = highLatencyNodes[i];
        const retestResult = retestResults[i];
        const originalIndex = testResults.findIndex(r => r.nodeId === node.id);
        
        if (originalIndex !== -1 && retestResult) {
          // 如果重测结果更好，使用重测结果
          if (retestResult.isSuccess && retestResult.latency && retestResult.latency <= 2000) {
            testResults[originalIndex] = retestResult;
          }
        }
      }
    }

    // 处理测试结果
    await this.processTestResults(nodes, testResults);

    // 生成订阅摘要
    const summary = this.generateSubscriptionSummary(nodes, testResults);

    return {
      subscription,
      nodes,
      testResults,
      summary
    };
  }

  /**
   * 生成订阅测试摘要
   */
  generateSubscriptionSummary(nodes, testResults) {
    const totalNodes = nodes.length;
    const onlineNodes = testResults.filter(r => r.isSuccess).length;
    const offlineNodes = totalNodes - onlineNodes;
    const successRate = totalNodes > 0 ? Math.round((onlineNodes / totalNodes) * 100) : 0;

    // 计算平均延迟和速度
    const successfulResults = testResults.filter(r => r.isSuccess && r.latency);
    const averageLatency = successfulResults.length > 0 
      ? Math.round(successfulResults.reduce((sum, r) => sum + r.latency, 0) / successfulResults.length)
      : null;

    const speedResults = testResults.filter(r => r.isSuccess && r.downloadSpeed);
    const averageSpeed = speedResults.length > 0
      ? Math.round((speedResults.reduce((sum, r) => sum + r.downloadSpeed, 0) / speedResults.length) * 100) / 100
      : null;

    // 获取故障节点列表
    const failedNodes = nodes.filter(node => {
      const result = testResults.find(r => r.nodeId === node.id);
      return result && !result.isSuccess;
    }).slice(0, 5); // 最多显示5个

    // 获取最佳节点列表
    const bestNodes = testResults
      .filter(r => r.isSuccess && r.latency)
      .sort((a, b) => a.latency - b.latency)
      .slice(0, 3)
      .map(result => {
        const node = nodes.find(n => n.id === result.nodeId);
        return {
          name: node ? node.name : 'Unknown',
          latency: result.latency
        };
      });

    return {
      totalNodes,
      onlineNodes,
      offlineNodes,
      successRate,
      averageLatency,
      averageSpeed,
      failedNodes,
      bestNodes,
      testTime: new Date().toLocaleString('zh-CN')
    };
  }

  /**
   * 发送订阅测试摘要
   */
  async sendSubscriptionTestSummary(subscriptionResult) {
    const { subscription, summary } = subscriptionResult;
    
    if (summary.totalNodes === 0) {
      return;
    }

    const title = `📊 订阅测试报告: ${escapeMarkdown(subscription.name)}`;
    
    const lines = [
      `*订阅名称:* ${escapeMarkdown(subscription.name)}`,
      `*测试时间:* ${escapeMarkdown(summary.testTime)}`,
      `*节点总数:* ${summary.totalNodes}个`,
      `*在线节点:* ${summary.onlineNodes}个`,
      `*离线节点:* ${summary.offlineNodes}个`,
      `*成功率:* ${summary.successRate}%`
    ];

    if (summary.averageLatency) {
      lines.push(`*平均延迟:* ${summary.averageLatency}ms`);
    }

    if (summary.averageSpeed) {
      lines.push(`*平均速度:* ${summary.averageSpeed}Mbps`);
    }

    if (summary.bestNodes.length > 0) {
      lines.push(`\n🏆 *最佳节点:*`);
      summary.bestNodes.forEach((node, index) => {
        lines.push(`${index + 1}. ${escapeMarkdown(node.name)} - ${node.latency}ms`);
      });
    }

    if (summary.failedNodes.length > 0) {
      lines.push(`\n❌ *故障节点:*`);
      summary.failedNodes.forEach((node, index) => {
        lines.push(`${index + 1}. ${escapeMarkdown(node.name)}`);
      });
    }

    const message = lines.join('\n');

    await this.notifier.sendSubscriptionNotification(subscription, title, message, summary);
  }

  /**
   * 发送总体测试摘要
   */
  async sendOverallTestSummary(subscriptionResults) {
    const totalSubscriptions = subscriptionResults.length;
    const totalNodes = subscriptionResults.reduce((sum, r) => sum + r.summary.totalNodes, 0);
    const totalOnlineNodes = subscriptionResults.reduce((sum, r) => sum + r.summary.onlineNodes, 0);
    const totalOfflineNodes = totalNodes - totalOnlineNodes;
    const overallSuccessRate = totalNodes > 0 ? Math.round((totalOnlineNodes / totalNodes) * 100) : 0;

    // 计算总体平均延迟
    const allSuccessfulResults = subscriptionResults.flatMap(r => 
      r.testResults.filter(result => result.isSuccess && result.latency)
    );
    const overallAverageLatency = allSuccessfulResults.length > 0
      ? Math.round(allSuccessfulResults.reduce((sum, r) => sum + r.latency, 0) / allSuccessfulResults.length)
      : null;

    const title = '📈 总体测试报告';
    const lines = [
      `*测试时间:* ${escapeMarkdown(new Date().toLocaleString('zh-CN'))}`,
      `*订阅数量:* ${totalSubscriptions}个`,
      `*节点总数:* ${totalNodes}个`,
      `*在线节点:* ${totalOnlineNodes}个`,
      `*离线节点:* ${totalOfflineNodes}个`,
      `*总体成功率:* ${overallSuccessRate}%`
    ];

    if (overallAverageLatency) {
      lines.push(`*总体平均延迟:* ${overallAverageLatency}ms`);
    }

    // 按订阅显示摘要
    lines.push('\n📋 *各订阅状态:*');
    subscriptionResults.forEach(result => {
      const { subscription, summary } = result;
      const status = summary.successRate >= 80 ? '🟢' : summary.successRate >= 50 ? '🟡' : '🔴';
      lines.push(`${status} ${escapeMarkdown(subscription.name)}: ${summary.onlineNodes}/${summary.totalNodes} (${summary.successRate}%)`);
    });

    const message = lines.join('\n');

    await this.notifier.sendSystemNotification(title, message, {
      totalSubscriptions,
      totalNodes,
      totalOnlineNodes,
      totalOfflineNodes,
      overallSuccessRate,
      overallAverageLatency
    });
  }

  /**
   * 处理测试结果
   */
  async processTestResults(nodes, testResults) {
    const nodeMap = new Map(nodes.map(node => [node.id, node]));
    const resultMap = new Map(testResults.map(result => [result.nodeId, result]));

    for (const node of nodes) {
      const testResult = resultMap.get(node.id);
      if (!testResult) continue;

      const previousStatus = node.status;
      const previousFailureCount = node.failureCount;

      // 更新节点状态
      if (testResult.isSuccess) {
        // 检查延迟是否超时（超过2000ms）
        const isTimeout = testResult.latency && testResult.latency > 2000;
        
        if (isTimeout) {
          // 延迟超时，标记为故障节点
          const newFailureCount = previousFailureCount + 1;
          
          await node.update({
            status: 'offline',
            lastTestTime: testResult.testTime,
            lastTestResult: false,
            failureCount: newFailureCount,
            averageLatency: testResult.latency
          });

          // 检查是否需要发送故障通知
          if (newFailureCount === this.failureThreshold && previousStatus === 'online') {
            await this.notifier.sendFailureNotification(node, testResult);
          }
          
          logger.warn(`节点延迟超时，标记为故障: ${node.name} - 延迟: ${testResult.latency}ms`);
        } else {
          // 测试成功且延迟正常
          if (node.status === 'offline') {
            // 节点从离线恢复
            if (previousFailureCount >= this.recoveryThreshold) {
              await this.notifier.sendRecoveryNotification(node, testResult);
            }
          }

          await node.update({
            status: 'online',
            lastTestTime: testResult.testTime,
            lastTestResult: true,
            failureCount: 0,
            averageLatency: testResult.latency,
            averageSpeed: testResult.downloadSpeed
          });
        }

      } else {
        // 测试失败
        const newFailureCount = previousFailureCount + 1;
        
        await node.update({
          status: 'offline',
          lastTestTime: testResult.testTime,
          lastTestResult: false,
          failureCount: newFailureCount
        });

        // 检查是否需要发送故障通知
        if (newFailureCount === this.failureThreshold && previousStatus === 'online') {
          await this.notifier.sendFailureNotification(node, testResult);
        }
      }
    }
  }

  /**
   * 清理旧数据
   */
  async cleanupOldData() {
    try {
      logger.info('开始清理旧数据');

      // 删除30天前的测试结果
      const thirtyDaysAgo = new Date();
      thirtyDaysAgo.setDate(thirtyDaysAgo.getDate() - 30);

      const deletedTestResults = await TestResult.destroy({
        where: {
          testTime: {
            [require('sequelize').Op.lt]: thirtyDaysAgo
          }
        }
      });

      // 删除7天前的通知记录
      const sevenDaysAgo = new Date();
      sevenDaysAgo.setDate(sevenDaysAgo.getDate() - 7);

      const deletedNotifications = await require('../models').Notification.destroy({
        where: {
          createdAt: {
            [require('sequelize').Op.lt]: sevenDaysAgo
          },
          isSent: true
        }
      });

      logger.info('旧数据清理完成', {
        deletedTestResults,
        deletedNotifications
      });

    } catch (error) {
      logger.error('清理旧数据失败', { error: error.message });
    }
  }

  /**
   * 手动触发测试
   */
  async triggerManualTest() {
    if (this.isRunning) {
      throw new Error('测试正在进行中，请稍后再试');
    }

    logger.info('手动触发速度测试');
    await this.runSpeedTest();
  }

  /**
   * 获取调度器状态
   */
  getStatus() {
    return {
      isRunning: this.isRunning,
      testInterval: this.testInterval,
      failureThreshold: this.failureThreshold,
      recoveryThreshold: this.recoveryThreshold,
      jobs: {
        speedTest: this.speedTestJob ? 'running' : 'stopped',
        notificationRetry: this.notificationRetryJob ? 'running' : 'stopped',
        cleanup: this.cleanupJob ? 'running' : 'stopped'
      }
    };
  }
}

module.exports = Scheduler; 